--
-- UART: main entity
--
-- Copyrighted by Wincent Balin
-- Idea by Jean P. Nicolle
--

library ieee;
use ieee.std_logic_1164.all;

entity UART is
	generic
	(
		frequency		: integer;
		baud			: integer
	);
	port
	(
		clk				: in std_logic;
		rxd				: in std_logic;
		txd				: out std_logic;
		txd_data		: in std_logic_vector(7 downto 0);
		txd_start		: in std_logic;
		txd_busy		: out std_logic;
		rxd_data		: out std_logic_vector(7 downto 0);
		rxd_data_ready	: out std_logic
	);
end entity UART;

architecture UARTArch of UART is
-- defining constants
								constant OVERSAMPLING : integer := 8;
-- defining signals
-- defining components
component UARTReceiver
	generic
	(
		frequency		: integer;
		baud			: integer;
		oversampling	: integer
	);
	port
	(
		clk				: in std_logic;
		rxd				: in std_logic;
		rxd_data		: out std_logic_vector(7 downto 0);
		rxd_data_ready	: out std_logic
	);
end component UARTReceiver;
component UARTTransmitter
	generic
	(
		frequency		: integer;
		baud			: integer
	);
	port
	(
		clk				: in std_logic;
		txd				: out std_logic;
		txd_data		: in std_logic_vector(7 downto 0);
		txd_start		: in std_logic;
		txd_busy		: out std_logic
	);
end component UARTTransmitter;
begin
	-- instantiating components
	RECEIVER : UARTReceiver
	generic map
	(
		frequency		=> frequency,
		baud			=> baud,
		oversampling	=> OVERSAMPLING
	)
	port map
	(
		clk				=> clk,
		rxd				=> rxd,
		rxd_data		=> rxd_data,
		rxd_data_ready	=> rxd_data_ready
	);
	TRANSMITTER : UARTTransmitter
	generic map
	(
		frequency		=> frequency,
		baud			=> baud
	)
	port map
	(
		clk				=> clk,
		txd				=> txd,
		txd_data		=> txd_data,
		txd_start		=> txd_start,
		txd_busy		=> txd_busy
	);
end UARTArch;
