--
-- UARTTest
--
-- Copyrighted by Wincent Balin
-- Idea by Jean P. Nicolle
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity UARTTest is
	port
	(
		clk	: in std_logic;
		rxd	: in std_logic;
		txd : out std_logic;
		led : out std_logic
	);
end entity UARTTest;

architecture UARTTestArch of UARTTest is
-- Input constants
										constant CLK_FREQUENCY : integer := 27000000;
										constant BAUD          : integer := 115200;
-- defining components
component UART
	generic
	(
		frequency		: integer;
		baud			: integer
	);
	port
	(
		clk				: in std_logic;
		rxd				: in std_logic;
		rxd_data		: out std_logic_vector(7 downto 0);
		rxd_data_ready	: out std_logic;
		txd				: out std_logic;
		txd_data		: in std_logic_vector(7 downto 0);
		txd_start		: in std_logic;
		txd_busy		: out std_logic
	);
end component UART;
-- defining signals
signal rxd_data_ready, txd_start, txd_busy : std_logic;
signal rxd_data, txd_data : std_logic_vector(7 downto 0);
--
type helloworld_type is (idle, h1, e1, l1, l2, o1, blank, w1, o2, r1, l3, d1, bang1, cr1);
signal state : helloworld_type := idle;
begin
	-- instantiating components
	SERIAL : UART
	generic map
	(
		frequency		=> CLK_FREQUENCY,
		baud			=> BAUD
	)
	port map
	(
		clk				=> clk,
		rxd				=> rxd,
		rxd_data		=> rxd_data,
		rxd_data_ready	=> rxd_data_ready,
		txd				=> txd,
		txd_data		=> txd_data,
		txd_start		=> txd_start,
		txd_busy		=> txd_busy
	);
	-- processes
	state_proc : process(clk)
	begin
		if clk'event and clk = '1'
		then
			case state is
				when idle =>
					txd_start <= '0';
					if rxd_data_ready = '1' and rxd_data = "00001101" then
						txd_start <= '1';
						state <= h1;
					end if;
				when h1 =>
					if txd_busy = '0' then
						txd_data <= "01001000";
						state <= e1;
					end if;	
				when e1 =>
					if txd_busy = '0' then
						txd_data <= "01100101";
						state <= l1;
					end if;
				when l1 =>
					if txd_busy = '0' then
						txd_data <= "01101100";
						state <= l2;
					end if;
				when l2 =>
					if txd_busy = '0' then
						txd_data <= "01101100";
						state <= o1;
					end if;
				when o1 =>
					if txd_busy = '0' then
						txd_data <= "01101111";
						state <= blank;
					end if;
				when blank =>
					if txd_busy = '0' then
						txd_data <= "00100000";
						state <= w1;
					end if;
				when w1 =>
					if txd_busy = '0' then
						txd_data <= "01110111";
						state <= o2;
					end if;
				when o2 =>
					if txd_busy = '0' then
						txd_data <= "01101111";
						state <= r1;
					end if;
				when r1 =>
					if txd_busy = '0' then
						txd_data <= "01110010";
						state <= l3;
					end if;
				when l3 =>
					if txd_busy = '0' then
						txd_data <= "01101100";
						state <= d1;
					end if;
				when d1 =>
					if txd_busy = '0' then
						txd_data <= "01100100";
						state <= bang1;
					end if;
				when bang1 =>
					if txd_busy = '0' then
						txd_data <= "00100001";
						state <= cr1;
					end if;
				when cr1 =>
					if txd_busy = '0' then
						txd_data <= "00001101";
						state <= idle;
					end if;
			end case;
		end if;
	end process state_proc;
	--
--	switch_txd_proc : process(clk)
--	begin
--		if clk'event and clk = '1' then
--			if state = idle then
--				txd_start <= '0';
--			else
--				txd_start <= '1';
--			end if;
--		end if;
--	end process switch_txd_proc;
	--
	led_proc : process(clk)
	begin
		if clk'event and clk = '1' then
			if state /= idle then
				led <= '1';
			else
				led <= '0';
			end if;
		end if;
	end process led_proc;
end architecture UARTTestArch;